%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lab 3: Equivalence Class Histograms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Qiang Qiu 
% qiang.qiu@duke.edu

% Given an arbitrary linear transformation y -> \phi y, the cyclic property 
% of the trace implies that

% w_{a,b}(\phi y) = 1/(2^{m/2} Tr[\phi yy' \phi' D(a,b)]
%                 = 1/(2^{m/2} Tr[yy' \phi' D(a,b)\phi],

% which in turn implies that \phi permutes the Weyl coefficient absolute values 
% if it is an outer automorphism of HW_{2^m}. The outer automorphism group of 
% the Heisenberg-Weyl group is known as the binary symplectic group.


%% Equivalence Class
% Let G be some group of symplectic transformations. We design equivalence 
% classes such that G can only permute the elements of HW_{2^m} within each 
% equivalence class. It follows that the average of the absolute values of 
% the Weyl coefficients within a given equivalence class is invariant under G.


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Exercise: 
% %%%%%%%%
% Given a vectorized N by N image patch y, taking G to be the group of 
% transformations generated by 90^o rotation, and cyclic horizontal and vertical 
% translation by any multiple of N/4. Design Equivalence classes for G.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




% %%%%%%%%
% Answer: 
% %%%%%%%%

% under 90^o rotation
% D([a1 a2], [b1 b2]) -> (-1)^{1'b1} D([a2 a1], [b2 b1])

% under multiple of N/4 cyclic horizontal and vertical translation
% D([a1 j k a2], [b1 l m b2]) -> (-1)^m D([a1 j+k k a2], [b1 l l+m b2])

% Consider 4 by 4 patches,  G partitions the a coordinates into the 6 
% equivalence classes:

% {(0 0 0 0)} , 
% {(1 0 1 0)} , 
% {(0 0 1 0), (1 0 0 0)},
% {(0 0 0 1), (0 0 1 1), (0 1 0 0), (1 1 0 0)} ,
% {(0 1 0 1), (0 1 1 1), (1 1 0 1), (1 1 1 1)} ,
% {(0 1 1 0), (1 0 0 1), (1 0 1 1), (1 1 1 0)} ,

% G also partitions the b coordinates into the 6 equivalence classes:
% {(0 0 0 0)} , 
% {(0 1 0 1)} , 
% {(0 0 0 1), (0 1 0 0)} ,
% {(0 0 1 0), (0 0 1 1), (1 0 0 0), (1 1 0 0)}
% {(1 0 1 0), (1 0 1 1), (1 1 1 0), (1 1 1 1)} ,
% {(1 0 0 1), (0 1 1 0), (0 1 1 1), (1 1 0 1)} .

% We thus partition the Weyl coefficients into 36 equivalence
% classes. Six of the classes contain Weyl coefficients which are all
% zero, since a'b = 1 for all members of the class. Thus we have thereby 
% reduced the number of Weyl coefficients from 256 to 24

% one example to show the invariance under 90 rotation
y = double(imread('img/1.jpg'));
y2= imrotate(y, 90);

y=y(:);
y2=y2(:);
 
[ws] = weyl_squashed(y);
[ws2] = weyl_squashed(y2);

figure; 
subplot(2,1,1); plot(1:length(ws(:)), ws(:));
subplot(2,1,2); plot(1:length(ws2(:)), ws2(:));
% %%%%%%%%

